package edu.uoc.pfc.mena.db.core.config;

import java.io.InputStream;
import java.util.Properties;

import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;


public class ConfiguradorFactory implements Constants {
	
	private static ConfiguradorFactory instance=null;	
	private Properties cfg=null;
	
	public static synchronized ConfiguradorFactory getInstance() throws BDException {
		  
	    if( instance == null ){
	    	instance = new ConfiguradorFactory();
	    	instance.init();
	   }    
	    
	    return instance;
	}
	
	 private void init() throws BDException {
		 // Obtenir les dades del properties
		 try{
		     InputStream is = getClass().getClassLoader().getResourceAsStream(CONFIG_FILE);
		     if (is==null) {
		    	 System.out.println("No es pot obtenir el fitxer de config ("+CONFIG_FILE+") del classloader, s'obte del current thread...");
		    	 ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
		    	 is = classLoader.getResourceAsStream(CONFIG_FILE);
		     }
		     cfg = new Properties();
		     cfg.load(is);
		 }catch (Exception e){
			 throw new BDException(Constants.CONFIGURATION_FILE_ERROR_CODE, Constants.CONFIGURATION_FILE_ERROR_MSG,e);
		 }
	 }
	
	 public String getDriverBD(){
		 return this.getProperty(DRIVER_BD);
	 }
	 
	 public String getIdDriverBD(){
		 return this.getProperty(ID_DRIVER_BD);
	 }
	 
	 
	 
	 public String getUrlBD(){
		 return this.getProperty(URL_BD);
	 }
	 
	 public String getUsuariBD(){
		 return this.getProperty(USER_BD);
	 }
	 
	 public String getContrasenyaBD(){
		 return this.getProperty(PASS_BD);
	 }
	 
	 public String getPackageBDBeans(){
		 return this.getProperty(PKG_BDBEANS);
	 }
	 
	 public String getBasedirBDBeans(){
		 return this.getProperty(BASEDIR_BDBEANS);
	 }
	 
	 public String getGeneratorPrefixPackageBeans(){
		 return this.getProperty(PREFIX_PACKAGE_GENERATOR);
	 }
	 
	 public String getGeneratorPrefixClasseBeans(){
		 return this.getProperty(PREFIX_CLASS_GENERATOR);
	 }
	 
	 public String getGeneratorSufixClasseBeans(){
		 return this.getProperty(SUFIX_CLASS_GENERATOR);
	 }
	 
	 
	 private String getProperty(String key){
		 return this.cfg.getProperty(key);
	 }
	 
}
