package edu.uoc.pfc.mena.db.core.pojo;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import edu.uoc.pfc.mena.db.core.annotations.Columna;
import edu.uoc.pfc.mena.db.core.annotations.Entitat;
import edu.uoc.pfc.mena.db.core.annotations.Id;
import edu.uoc.pfc.mena.db.core.annotations.Restriccio;
import edu.uoc.pfc.mena.db.core.annotations.Taula;
import edu.uoc.pfc.mena.db.core.annotations.ValorGenerat;
import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.db.core.util.SqlHelper;

public class Registre {

	private Object objecte;
	private String taula;
	private List<edu.uoc.pfc.mena.db.core.pojo.Columna> columnes;
	private String clauPrimaria;
	
//	private edu.uoc.pfc.mena.db.core.pojo.Restriccio restriccio;
	
	public Registre(){
		this.columnes= new ArrayList<edu.uoc.pfc.mena.db.core.pojo.Columna>();
	}
	
	public Registre(Object obj) throws BDException{
		this();
		this.objecte=obj;
		this.getRegitreFromObject();
	}
	
	public Object getObjecte() {
		return objecte;
	}
	
	public void setObjecte(Object objecte) {
		this.objecte=objecte;
	}
	
	public String getTaula() {
		return taula;
	}

	public void setTaula(String taula) {
		this.taula = taula;
	}

	public List<edu.uoc.pfc.mena.db.core.pojo.Columna> getColumnes() {
		return columnes;
	}

	public void setColumnes(List<edu.uoc.pfc.mena.db.core.pojo.Columna> columnes) {
		this.columnes = columnes;
	}

	
//	public String getSequencia() {
//		return sequencia;
//	}
//
//	public void setSequencia(String sequencia) {
//		this.sequencia = sequencia;
//	}

	public String getClauPrimaria() {
		return clauPrimaria;
	}

	public void setClauPrimaria(String clauPrimaria) {
		this.clauPrimaria = clauPrimaria;
	}

	
	
	public Registre getRegitreFromObject() throws BDException {
		
		if (this.objecte==null){
			throw new BDException(Constants.REGISTRE_NULL_OBJECT_ERROR_CODE, Constants.REGISTRE_NULL_OBJECT_ERROR_MSG);
		}
		else {
			parse(this.objecte);
		} 
	    
	    return this;
	}
	
	private void parse(Object obj) throws BDException {
		Class<?> clazz=obj.getClass(); 
		if (!clazz.isAnnotationPresent(Entitat.class)){
			throw new BDException(Constants.REGISTRE_NO_ENTITAT_ANNOTATION_ERROR_CODE, Constants.REGISTRE_NO_ENTITAT_ANNOTATION_ERROR_MSG);
		} 
		if (clazz.isAnnotationPresent(Taula.class)) {
			Taula taula = clazz.getAnnotation(Taula.class);
	        this.taula=taula.nom();
		} else {
			this.taula=SqlHelper.getDefaultTableName(clazz.toString());
		}
		
        Method[] methods = clazz.getMethods();

	    for (Method method : methods) {
	    	// Cas d'una columna
	    	edu.uoc.pfc.mena.db.core.pojo.Columna col= null;
	        if (method.isAnnotationPresent(Columna.class)) {
	        	Columna column = method.getAnnotation(Columna.class);
	        	col= new edu.uoc.pfc.mena.db.core.pojo.Columna();
	        	col.setNom(column.nom());
	        	col.setTipus(column.tipus());
	        	col.setMetode(method.getName());
	        	col.setClauPrimaria(false);
	        }   
	        if (method.isAnnotationPresent(Id.class)) {
	        	Id id = method.getAnnotation(Id.class);
	        	this.clauPrimaria=id.nom();
	        	col=new edu.uoc.pfc.mena.db.core.pojo.Columna();
	        	col.setNom(id.nom());
	        	col.setTipus(id.tipus());
	        	col.setMetode(method.getName());
	        	col.setClauPrimaria(true);
	        }
	        
	        if (method.isAnnotationPresent(ValorGenerat.class)) {
	        	ValorGenerat vg = method.getAnnotation(ValorGenerat.class);
	        	col.setSequencia(vg.clau());
	        }
	        
	        // Finalment afegim la columna amb la info al llistat
	        if (col!=null){
	        	// Afegim la columna
	        	this.columnes.add(col);
	        }
	        
	    } 
	} 
	
	public edu.uoc.pfc.mena.db.core.pojo.Columna getColumnaClauPrimaria(){
		
		edu.uoc.pfc.mena.db.core.pojo.Columna col= new edu.uoc.pfc.mena.db.core.pojo.Columna();
		for (edu.uoc.pfc.mena.db.core.pojo.Columna c:columnes){
			if (c.isClauPrimaria()) {
				col.setClauPrimaria(true);
				col.setMetode(c.getMetode());
				col.setNom(c.getNom());
				col.setSequencia(c.getSequencia());
				col.setTipus(c.getTipus());
				return col; // TODO: revisar per multiples columnes
			}
		}
		return col;
	}
}
