package edu.uoc.pfc.mena.db.core.session;

import java.sql.ResultSet;
import java.util.List;

import org.apache.log4j.Logger;

import edu.uoc.pfc.mena.db.core.connection.factory.ConnexioFactory;
import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.db.core.pojo.Registre;
import edu.uoc.pfc.mena.db.core.transaction.Transaccio;
import edu.uoc.pfc.mena.db.core.transaction.TransaccioImpl;
import edu.uoc.pfc.mena.db.core.util.SqlHelper;
import edu.uoc.pfc.mena.db.core.util.Utils;

public class SessioImpl implements Sessio, Constants{
	private static Logger log = Logger.getLogger(SessioImpl.class.getName());
	
	Transaccio trans=null;
	
	public SessioImpl(){
	}

	public Transaccio getTransaccio() {
		return trans;
	}
	
	public Transaccio newTransaccio() throws BDException{
		return newTransaccio(true);
	}
	
	public Transaccio newTransaccio(boolean autocommit) throws BDException {
		try{
			this.trans = new TransaccioImpl(autocommit);
			return this.trans;
		}catch (Exception e){
			log.error(e.getMessage()); 
			throw new BDException(Constants.TRANSACTION_ERROR_CODE, Constants.TRANSACTION_ERROR_MSG+ e.getMessage(), e);
		}
	}

	public void setTransaccio(Transaccio trans) {
		this.trans = trans;
	}

	
	public int insert(Object objecte) throws BDException {
		
		int idAutoInc=-1;
		//  Recuperar l'objecte SQL per reflexion a partir de la classe i de les annotacions
		Registre reg= new Registre(objecte);
			
		String sql=SqlHelper.getInstance().toInsert(reg);
		if (ConnexioFactory.getInstance().getConnexio().getUrl().indexOf(ORACLE_TEXT)>=0){
			// Autoincremental per a Oracle
			idAutoInc = this.getTransaccio().executaCount("SELECT "+reg.getClauPrimaria()+".nextval FROM DUAL");
			sql=Utils.replaceString(sql,ORACLE_ID_AUTO_INCREMENTAL,""+idAutoInc);
			doExecute(sql);
		} else if (ConnexioFactory.getInstance().getConnexio().getUrl().indexOf(MYSQL_TEXT)>=0){
			// Autoincremental MySQL
			doExecute(sql);
			idAutoInc = this.getTransaccio().executaCount("SELECT LAST_INSERT_ID()") +1;
		} else {
			doExecute(sql);
		}	
		return idAutoInc;
	}
	
	@Override
	public void update(Object objecte) throws BDException {
		//  Recuperar l'objecte SQL per reflexion a partir de la classe i de les annotacions
		Registre reg= new Registre(objecte);
		String sql=SqlHelper.getInstance().toUpdate(reg);
		// Executar l'script d'insert
		doExecute(sql);
	}

	@Override
	public void delete(Object objecte) throws BDException {
		//  Recuperar l'objecte SQL per reflexion a partir de la classe i de les annotacions
		Registre reg= new Registre(objecte);
		String sql=SqlHelper.getInstance().toDelete(reg);
		// Executar l'script d'insert
		doExecute(sql);
	}
	
	public List<Object> find(Object objecte) throws BDException {
		Registre reg= new Registre(objecte);
		String sql=SqlHelper.getInstance().toFindById(reg);
	    ResultSet rs=doExecuteQuery(sql);
	    return SqlHelper.resultSetToObject(rs, objecte);		        
	}

	public void tancaTransaccio() throws BDException {
		this.trans.tanca();
		this.trans=null;
	}
	
	public List<Object> find(String sql, Object returnObj) throws BDException {
	    ResultSet rs=doExecuteQuery(sql);
	    return SqlHelper.resultSetToObject(rs, returnObj);		
	}
	
	/**
	 * Metodes privats 
	 * @param sql
	 * @throws BDException
	 */
	private void doExecute(String sql) throws BDException {
		trans.executa(sql);
	}
	
	private ResultSet doExecuteQuery(String sql) throws BDException {
		return trans.executaQuery(sql);
	}
}
