package edu.uoc.pfc.mena.db.core.transaction;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;

import edu.uoc.pfc.mena.db.core.connection.factory.ConnexioFactory;
import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;

public class TransaccioImpl implements Transaccio, Constants{
	private static Logger log = Logger.getLogger(TransaccioImpl.class.getName());
	private Connection connexio;

	public TransaccioImpl(boolean autoCommit) throws BDException{
		
		try {
			connexio=ConnexioFactory.getInstance().creaConnexio();
			connexio.setAutoCommit(autoCommit);
		} catch (SQLException e) {
			throw new BDException(TRANSACTION_CONNECTION_ERROR_CODE, TRANSACTION_CONNECTION_ERROR_MSG, e);
		}
	}
		
	public void commit() throws BDException {
		if (this.connexio==null) {
			throw new BDException(NULL_CONNECTION_ERROR_CODE,NULL_CONNECTION_ERROR_MSG+": (commit)");
		}
		try{
			this.connexio.commit();
		}catch(Exception e){
			throw new BDException(COMMIT_ERROR_CODE,COMMIT_ERROR_MSG+ e.getMessage(), e);
		}
	}
	
	public void rollback() throws BDException {
		if (this.connexio==null) {
			throw new BDException(NULL_CONNECTION_ERROR_CODE,NULL_CONNECTION_ERROR_MSG+": (rollback)");
		}
		try{
			this.connexio.rollback();
		} catch (Exception e) {
			throw new BDException(ROLLBACK_ERROR_CODE,ROLLBACK_ERROR_MSG + e.getMessage(), e);
		}
	}
	
	public void tanca() throws BDException {
		try{
			// Es deixa l autocommit al valor per defecte
			if (!this.connexio.getAutoCommit()){
				this.connexio.setAutoCommit(true);
			}
			if (this.connexio!=null){
				this.connexio.close();
			}
		}catch(Exception e){
			log.error(e.getMessage()); 
			throw new BDException(CLOSE_ERROR_CODE,CLOSE_ERROR_MSG + e.getMessage(), e);
		}
		finally{
			this.connexio=null;
		}
	}

	public void executa(String sql) throws BDException {
		try{
			// Executar l'insert
			Statement st=this.connexio.createStatement();
		    if (st.executeUpdate(sql) != 1) {
		        throw new BDException(EXECUTING_UPDATE_MULTIPLE_REGISTRES_ERROR_CODE,EXECUTING_UPDATE_MULTIPLE_REGISTRES_ERROR_MSG);
		    } 
		} catch(Exception e){ 
			throw new BDException(EXECUTING_UPDATE_ERROR_CODE,EXECUTING_UPDATE_ERROR_MSG + e.getMessage(), e);
		}
	}
		

	public ResultSet executaQuery(String sql) throws BDException {
		try{
			// Executar l'insert
			Statement stm=this.connexio.createStatement();
		    return stm.executeQuery(sql);
		}catch(Exception e){
			throw new BDException(EXECUTING_QUERY_ERROR_CODE,EXECUTING_QUERY_ERROR_MSG + e.getMessage(), e);
		}
	}
	
	/**
	 * 
	 */
	public int executaCount(String sql) throws BDException {
		try{
			// Executar l'insert
			Statement stm=this.connexio.createStatement();
			ResultSet  rs = stm.executeQuery(sql);
		    rs.next();
		    int id = rs.getInt(1);
		    return id;
		}catch(Exception e){
			throw new BDException(EXECUTING_QUERY_COUNT_ERROR_CODE,EXECUTING_QUERY_COUNT_ERROR_MSG + e.getMessage(), e);
		}
	}
		
}
