package edu.uoc.pfc.mena.db.core.util;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.Logger;

import edu.uoc.pfc.mena.db.core.annotations.Columna;
import edu.uoc.pfc.mena.db.core.annotations.Entitat;
import edu.uoc.pfc.mena.db.core.annotations.Id;
import edu.uoc.pfc.mena.db.core.annotations.Mapper;
import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;


public class ResultSetMapper<T> {

	private static Logger log = Logger.getLogger(ResultSetMapper.class.getName());
	
	public List<T> mapRersultSetToObject(ResultSet rs, Class outputClass) throws BDException {
		List<T> outputList = null;
		try {
			// make sure resultset is not null
			if (rs != null) {
				// comprovacio que la classe de sortida es una Entitat (querys sobre una taula) o un Mapper( per a fer querys amb multiples taules)
				if (outputClass.isAnnotationPresent(Entitat.class) || outputClass.isAnnotationPresent(Mapper.class)) {
					// s'obte la metadata del resultset
					ResultSetMetaData rsmd = rs.getMetaData();
					// get all the attributes of outputClass
					Method[] methods = outputClass.getMethods();
					while (rs.next()) {
						T bean = (T) outputClass.newInstance();
						for (int _iterator = 0; _iterator < rsmd.getColumnCount(); _iterator++) {
							// s'obte el nom de la columna SQL (l'etiqueta)
							String columnName = rsmd.getColumnLabel(_iterator + 1); // S'obte el nom de l'alias
							// S'obte el valor de la columna SQL
							Object columnValue = rs.getObject(_iterator + 1);
							
							for (Method method : methods) {
								if (method.isAnnotationPresent(Columna.class) || method.isAnnotationPresent(Id.class)) {
									// Obtenim l'atribut associat al getter
									String attribute=method.getName().substring(3,4).toLowerCase().concat(method.getName().substring(4));
									// cas que es tracti d'una columna
									Columna column = method.getAnnotation(Columna.class);
									if (column !=null && column.nom().equalsIgnoreCase(columnName) && columnValue != null) {
										BeanUtils.setProperty(bean, attribute, columnValue);
										log.debug(">>> S'afegeix el valor"+columnValue+ "a l'atrubut: "+attribute);
										break;
									}
									// Cas que es tracti de la columna que es la clau primaria
									Id id = method.getAnnotation(Id.class);
									if (id!=null && id.nom().equalsIgnoreCase(columnName) && columnValue != null) {
										BeanUtils.setProperty(bean, attribute, columnValue);
										log.debug(">>> S'afegeix el valor"+columnValue+ "a l'atrubut: "+attribute);
										break;
									}
								}
							}
						}
						if (outputList == null) {
							outputList = new ArrayList<T>();
						}
						outputList.add(bean);
					}

				} else {
					// Error:  no es una classe de tipus Entitat ni Mapper
					throw new BDException (Constants.ANNOTATION_CLASS_MAPPING_ERROR_CODE, Constants.ANNOTATION_CLASS_MAPPING_ERROR_MSG);
				}
			} else {
				return null;
			}
		} catch (IllegalAccessException e) {
			throw new BDException (Constants.ACCES_MAPPING_ERROR_CODE, Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		} catch (SQLException e) {
			throw new BDException (Constants.SQL_MAPPING_ERROR_CODE, Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		} catch (InstantiationException e) {
			throw new BDException (Constants.INSTANTATION_MAPPING_ERROR_CODE, Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		} catch (InvocationTargetException e) {
			throw new BDException (Constants.INVOCATION_MAPPING_ERROR_CODE, Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		}

		return outputList;
	}
}
