package edu.uoc.pfc.mena.db.core.util;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import edu.uoc.pfc.mena.db.core.connection.factory.ConnexioFactory;
import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.db.core.pojo.Columna;
import edu.uoc.pfc.mena.db.core.pojo.Registre;
import edu.uoc.pfc.mena.db.utilities.GeneradorConstants;

public class SqlHelper implements Constants, GeneradorConstants  {

	private static Logger log = Logger.getLogger(SqlHelper.class.getName());
	private static final HashMap<String,String> sqlTokens;
	private static Pattern sqlTokenPattern;

	static
	{           
	    String[][] search_regex_replacement = new String[][]
	    {
	    		//search string     search regex        sql replacement regex
	            {   "\u0000"    ,       "\\x00"     ,       "\\\\0"     },
	            {   "'"         ,       "'"         ,       "\''"     },
	            {   "\""        ,       "\""        ,       "\\\\\""    },
	            {   "\b"        ,       "\\x08"     ,       "\\\\b"     },
	            {   "\n"        ,       "\\n"       ,       "\\\\n"     },
	            {   "\r"        ,       "\\r"       ,       "\\\\r"     },
	            {   "\t"        ,       "\\t"       ,       "\\\\t"     },
	            {   "\u001A"    ,       "\\x1A"     ,       "\\\\Z"     },
	            {   "\\"        ,       "\\\\"      ,       "\\\\\\\\"  }
	    };

	    sqlTokens = new HashMap<String,String>();
	    String patternStr = "";
	    for (String[] srr : search_regex_replacement)
	    {
	        sqlTokens.put(srr[0], srr[2]);
	        patternStr += (patternStr.isEmpty() ? "" : "|") + srr[1];            
	    }
	    sqlTokenPattern = Pattern.compile('(' + patternStr + ')');
	}
	
	private static SqlHelper instance;
	
	private SqlHelper(){
		super();
	}
	
	public static synchronized SqlHelper getInstance(){
		if (instance==null){
			instance=new SqlHelper();
		}
		return instance;
	}
	
	/**
	 * 
	 * @param reg
	 * @return
	 * @throws BDException
	 */
	public String toInsert(Registre reg) throws BDException {
		
		try{
			String values="";
			StringBuffer strBuffer= new StringBuffer();
			strBuffer.append(INSERT_PREFIX_TEXT);
			strBuffer.append(reg.getTaula().toUpperCase());
			
			int i=0;
			for (Columna col:reg.getColumnes()){
				
				// Check si es PK i AutoGen
				if (col.isClauPrimaria() && !"".equals(col.getSequencia())){
					if (ConnexioFactory.getInstance().getConnexio().getUrl().indexOf(ORACLE_TEXT)>=0){
						// Estem davant una clau primaria autogenerada 
						if (i>0){
							strBuffer.append(COMMA);
							values=values.concat(COMMA);
						}
						else {
							strBuffer.append(PARENTESIS_OPEN);
						}
							
						strBuffer.append(col.getNom().toUpperCase());
						i++;
						values=values.concat(ORACLE_ID_AUTO_INCREMENTAL);
					}
				}
				else {
				
					// Cas especific 
					if (i>0){
						strBuffer.append(COMMA);
						values=values.concat(COMMA);
					}
					else {
						strBuffer.append(PARENTESIS_OPEN);
					}
					
					strBuffer.append(col.getNom().toUpperCase());
					i++;
					
					Method method =reg.getObjecte().getClass().getMethod(col.getMetode(), null);
					Object valorAsObj = (Object)method.invoke(reg.getObjecte(), null);
					String result="";

					String javaTypeStr = getTipusJava(col.getTipus());
					
					if (valorAsObj !=null) {
						if (!valorAsObj.getClass().getName().equalsIgnoreCase(javaTypeStr)){
							throw new BDException(TYPE_MAPPING_ERROR_CODE,"Els tipus de la columna "+col.getNom()+" no coincideixen. S'ha trobat "+valorAsObj.getClass().getName()+") quan s'esperava "+javaTypeStr+"!");
						}
						
						if (valorAsObj instanceof java.lang.String) {
							result=escapeString((String)valorAsObj);
							result="'"+ result+ "'";
						}
						else if (valorAsObj instanceof java.sql.Timestamp){
							result="'"+ valorAsObj+ "'";
						}
						else {
							result=""+valorAsObj;
						}
						values=values.concat(result);
					} else {
						values=values+"null";
					}
				}
			}
			strBuffer.append(VALUES_TEXT);
			strBuffer.append(values);
			strBuffer.append(PARENTESIS_CLOSE);
			log.debug(">>> Query : "+strBuffer.toString());
			return strBuffer.toString();
		
		}catch(IllegalArgumentException e){
			throw new BDException(Constants.ARGUMENT_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(IllegalAccessException e){
			throw new BDException(Constants.ACCES_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(InvocationTargetException e){
			throw new BDException(Constants.INVOCATION_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(SecurityException e){
			throw new BDException(Constants.SECURITY_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(NoSuchMethodException e){
			throw new BDException(Constants.METHOD_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		}
	}
	
	/**
	 * 
	 * @param reg
	 * @return
	 * @throws Exception
	 */
	public String toUpdate(Registre reg) throws BDException {
		
		try {
			StringBuffer strBuffer= new StringBuffer();
			strBuffer.append(UPDATE_PREFIX_TEXT);
			strBuffer.append(reg.getTaula().toUpperCase());
			strBuffer.append(SET_TEXT);
			int i=0;
			String whereClause=WHERE_TEXT;
			for (Columna col:reg.getColumnes()){
						
				Method method =reg.getObjecte().getClass().getMethod(col.getMetode(), null);
				Object valorAsObj = (Object)method.invoke(reg.getObjecte(), null);
				String result="";
					
				String javaTypeStr = getTipusJava(col.getTipus());
				if (valorAsObj!=null){
					
					// Cas especific 
					if (i>0){
						strBuffer.append(COMMA);
					}
							
					
					if (!valorAsObj.getClass().getName().equalsIgnoreCase(javaTypeStr)){
						String errorMsg="Els tipus de la columna "+col.getNom()+"["+col.getTipus()+"] no coincideixen. S'ha trobat "+valorAsObj.getClass().getName()+" quan s'esperava "+javaTypeStr+"!";
						throw new BDException(TYPE_MAPPING_ERROR_CODE,errorMsg);
					}
						
					if (valorAsObj instanceof java.lang.String) {
						result=escapeString((String)valorAsObj);
						result="'"+ result+ "'";
					}
					else if (valorAsObj instanceof java.sql.Timestamp){
						result="'"+ valorAsObj+ "'";
					}
					else {
						result=""+valorAsObj;
					}
					
					// En cas que sigui la clau primaria sera el where de la execucio
					if (col.isClauPrimaria()){
						whereClause+=col.getNom()+EQUALS+result;
					}
					else {
						strBuffer.append(col.getNom());
						strBuffer.append(EQUALS);
						strBuffer.append(result);
						i++;
					}
				}
			}
			strBuffer.append(whereClause);
			log.debug(">>> Query : "+strBuffer.toString());
			return strBuffer.toString();
			
		}catch(IllegalArgumentException e){
			throw new BDException(Constants.ARGUMENT_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(IllegalAccessException e){
			throw new BDException(Constants.ACCES_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(InvocationTargetException e){
			throw new BDException(Constants.INVOCATION_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(SecurityException e){
			throw new BDException(Constants.SECURITY_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(NoSuchMethodException e){
			throw new BDException(Constants.METHOD_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		}
	}
	
	/**
	 * 
	 * @param reg
	 * @return
	 * @throws Exception
	 */
	public String toDelete(Registre reg) throws BDException {

		try {
			StringBuffer strBuffer= new StringBuffer();
			strBuffer.append(DELETE_PREFIX_TEXT);
			strBuffer.append(reg.getTaula().toUpperCase());
			
			for (Columna col:reg.getColumnes()){
				
				// Check si es PK i AutoGen
				if (col.isClauPrimaria()){
					strBuffer.append(WHERE_TEXT);
					strBuffer.append(col.getNom().toUpperCase());
					strBuffer.append(EQUALS);
					
					Method method =reg.getObjecte().getClass().getMethod(col.getMetode(), null);
					Object valorAsObj = (Object)method.invoke(reg.getObjecte(), null);
					String result="";
					String javaTypeStr = getTipusJava(col.getTipus());					
					if (!valorAsObj.getClass().getName().equalsIgnoreCase(javaTypeStr)){
						throw new BDException(TYPE_MAPPING_ERROR_CODE,"Els tipus de la columna "+col.getNom()+" no coincideixen. S'ha trobat "+valorAsObj.getClass().getName()+") quan s'esperava "+javaTypeStr+"!");
					}
					
					if (valorAsObj instanceof java.lang.String) {
						result=escapeString((String)valorAsObj);
						result="'"+ result+ "'";
					}
					else if (valorAsObj instanceof java.sql.Timestamp){
						result="'"+ valorAsObj+ "'";
					}
					else {
						result=""+valorAsObj;
					}
					strBuffer.append(result);
				}
			}
			log.debug(">>> Query : "+strBuffer.toString());
			return strBuffer.toString();
		}catch(IllegalArgumentException e){
			throw new BDException(Constants.ARGUMENT_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(IllegalAccessException e){
			throw new BDException(Constants.ACCES_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(InvocationTargetException e){
			throw new BDException(Constants.INVOCATION_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(SecurityException e){
			throw new BDException(Constants.SECURITY_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(NoSuchMethodException e){
			throw new BDException(Constants.METHOD_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		}
	}
	
	public String toFindById (Registre reg) throws BDException {
		StringBuffer strBuffer= new StringBuffer();
		strBuffer.append(SELECT_PREFIX_TEXT);
		strBuffer.append(reg.getTaula().toUpperCase());
		appendPrimaryKey(strBuffer, reg);	
			
		return strBuffer.toString();
	}
	

	private void appendPrimaryKey(StringBuffer strBuffer,Registre reg) throws BDException {
		try {
				Columna col= reg.getColumnaClauPrimaria();
				// Check si es PK i AutoGen
				strBuffer.append(WHERE_TEXT);
				strBuffer.append(col.getNom());
				strBuffer.append(EQUALS);
				
				Method method = reg.getObjecte().getClass().getMethod(col.getMetode(), null);
				
				Object valorAsObj = (Object)method.invoke(reg.getObjecte(), null);
				String result="";
				
				String javaTypeStr = getTipusJava(col.getTipus());
				
				if (!valorAsObj.getClass().getName().equalsIgnoreCase(javaTypeStr)){
					throw new BDException(Constants.TYPE_MAPPING_ERROR_CODE,"Els tipus de la columna "+col.getNom()+" i el bean no coincideixen. S'ha trobat "+valorAsObj.getClass().getName()+") quan s'esperava "+javaTypeStr+"!");
				}
				
				if (valorAsObj instanceof java.lang.String) {
					result=escapeString((String)valorAsObj);
					result="'"+ result+ "'";
				}
				else if (valorAsObj instanceof java.sql.Timestamp){
					result="'"+ valorAsObj+ "'";
				}
				else {
					result=""+valorAsObj;
				}
				
				strBuffer.append(result);
				
		}catch(IllegalArgumentException e){
			throw new BDException(Constants.ARGUMENT_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(IllegalAccessException e){
			throw new BDException(Constants.ACCES_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(InvocationTargetException e){
			throw new BDException(Constants.INVOCATION_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(SecurityException e){
			throw new BDException(Constants.SECURITY_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(),e);
		}catch(NoSuchMethodException e){
			throw new BDException(Constants.METHOD_MAPPING_ERROR_CODE,Constants.MAPPING_ERROR_MSG + e.getMessage(), e);
		}
	}
	
	
	public static String getDefaultTableName (String className){
		return className;
	}
	
	
	public static List<Object> resultSetToObject(ResultSet rs, Object pojo) throws BDException{
		Class<?> clazz=pojo.getClass();
		ResultSetMapper<Object> resultSetMapper = new ResultSetMapper<Object>();
		return resultSetMapper.mapRersultSetToObject(rs, clazz);
	}
	
	public static String getTipusJavaShort(String nomTipus) {
		String tipusJava=getTipusJava(nomTipus);
		return tipusJava.substring(tipusJava.lastIndexOf("."));
	}
	
	public static String getTipusJava(String nomTipus) {
		
		try {
			if ("BINARY".equals (nomTipus) || "VARBINARY".equals (nomTipus) || "TINYBLOB".equals (nomTipus) || "MEDIUMBLOB".equals (nomTipus) || "LONGBLOB".equals (nomTipus) ){
				return JAVA_TYPE_BYTE_ARRAY;
			} else if ("SMALLINT".equals (nomTipus) || "MEDIUMINT".equals (nomTipus) || "INT".equals (nomTipus) || "INTEGER".equals (nomTipus) ){
					return JAVA_TYPE_INTEGER;
			} else if("INTEGER UNSIGNED".equals (nomTipus) || "BIGINT".equals (nomTipus)){
				return JAVA_TYPE_LONG;
			} else if("FLOAT".equals (nomTipus)){
				return JAVA_TYPE_FLOAT;
			} else if("DOUBLE".equals (nomTipus)){
				return JAVA_TYPE_DOUBLE;
			} else if("DATETIME".equals (nomTipus) || "TIMESTAMP".equals (nomTipus) ) {
				return JAVA_TYPE_TIMESTAMP;
			} else if("DECIMAL".equals (nomTipus) || "TIMESTAMP".equals (nomTipus) ) {
				return JAVA_TYPE_DECIMAL;
			} else if("BIT".equals (nomTipus)|| ("TINYINT".equals (nomTipus))) {
				return JAVA_TYPE_BOOLEAN;
			} else { return JAVA_TYPE_STRING;}	
		}catch(Exception e){
			return JAVA_TYPE_STRING;
		}
	}
	
	public String escapeString(String valorAsObj) {

		 Matcher matcher = sqlTokenPattern.matcher(valorAsObj);
		    StringBuffer sb = new StringBuffer();
		    while(matcher.find())
		    {
		        matcher.appendReplacement(sb, sqlTokens.get(matcher.group(1)));
		    }
		    matcher.appendTail(sb);
		    

		    return sb.toString();
	}
	
}
