package edu.uoc.pfc.mena.db.utilities;

import java.io.BufferedWriter;
import java.io.File;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

import edu.uoc.pfc.mena.db.core.config.ConfiguradorFactory;
import edu.uoc.pfc.mena.db.core.connection.factory.ConnexioFactory;
import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;

public class GeneradorBeans implements GeneradorConstants {

	private static GeneradorBeans instance= null;

	Connection con= null;
	
	private GeneradorBeans(){
	}
	
	public static synchronized GeneradorBeans getInstance() throws BDException {
		if (instance==null){
			instance=new GeneradorBeans();
			instance.con=ConnexioFactory.getInstance().creaConnexio();
		}
		return instance;
	}
	
	public void generaBDBeans() throws BDException{
		
		// Obte les propietats on es generaran les classes
		
		BufferedWriter out=null;
		try {
			
			String directoriBase=ConfiguradorFactory.getInstance().getBasedirBDBeans();
			String nomPackage= ConfiguradorFactory.getInstance().getPackageBDBeans();

			for(String taula: getTaulesEsquemaList(con)){
				GeneradorTaulaBDInfo gt=(GeneradorTaulaBDInfo) getDadesFromSchema(ConnexioFactory.getInstance().getConnexio().getIdControlador(), taula);	
				// s'esborra el directori
				File folderBase=new File(GeneradorUtils.getFolderBase(directoriBase, nomPackage));
				GeneradorUtils.deleteFolder(folderBase);
				
				// Es crea el fitxer d'utilitats
				BufferedWriter out2=GeneradorUtils.createFile(folderBase, BDUTILS_CLASS_FILE);
				GeneradorUtils.writeHeader(out2, nomPackage, BDUTILS_CLASS_NAME);
				GeneradorUtils.writeConstructor(out2, BDUTILS_CLASS_NAME);
				GeneradorUtils.writeMetodesUtils(out2);
				GeneradorUtils.writeEnd(out2);
				
				// Creem els POJOs per a cadascuna de les taules
				String nomFitxer=gt.getNomFitxer();
				
				gt.addPrimaryKey();
				gt.addColumnesInfo();
				
				// Es crea el fitxer
				out=GeneradorUtils.createFile(folderBase, nomFitxer);
				
				// S'escriu la classe java en funcio de la BBDD
				GeneradorUtils.writeHeader(out, gt);
				GeneradorUtils.writeConstructor(out, gt);
				GeneradorUtils.writeSetGet(out, gt);
				GeneradorUtils.writeEnd(out);
			}
		}catch (Exception e){
			throw new BDException(Constants.GENERADOR_BEANS_ERROR_CODE, Constants.GENERADOR_BEANS_ERROR_MSG+ e.getMessage(), e);
		} finally{
			try{out.close();}catch(Exception ignored){}
		}
	}
	
	private GeneradorTaulaBDInfo getDadesFromSchema(String idDriver, String nomTaula) throws Exception {
		
		// Fer  la creacio de la taula per refection en funcio de la configuracio
		GeneradorTaulaBDInfo gd= (GeneradorTaulaBDInfo) Class.forName(GeneradorUtils.getClassNameFromDriver(idDriver)).newInstance();
		gd.setNomTaula(nomTaula);
		gd.setNomFitxer(toFileName(nomTaula));
		gd.setNomClasse(GeneradorUtils.toCamelCase(nomTaula,false));
		return gd;
	}

	public static void main (String args[]) throws Exception{
		System.out.println(">> Creant els beans d'acces a la capa de dades...");
		GeneradorBeans.getInstance().generaBDBeans();
		System.out.println(">> Fet!!!");
	}
	
	private String toFileName(String taula) {
		return GeneradorUtils.toCamelCase(taula,false).concat(".java");
	}
	
	private String toMethodName(String taula) {
		return GeneradorUtils.toCamelCase(taula,true);
	}
	
	
	private List<String> getTaulesEsquemaList(Connection con) throws Exception{
		
		List<String> taules= new ArrayList<String>();
		DatabaseMetaData meta = con.getMetaData();
	    ResultSet res = meta.getTables(null, null, null, new String[] {"TABLE"});
	    System.out.println(">>>> Llista de taules disponibles : "); 
	    while (res.next()) {
	    	  String tableName=res.getString("TABLE_NAME");
	    	  System.out.println(">>>>   "+res.getString("TABLE_CAT") + ", "+res.getString("TABLE_SCHEM") + ", "+tableName + ", "+res.getString("TABLE_TYPE") + ", "+res.getString("REMARKS"));
	         taules.add(tableName);
	      }
	      
	      return taules;
	}
	
	
}
