package edu.uoc.pfc.mena.db.utilities;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;

import edu.uoc.pfc.mena.db.core.config.ConfiguradorFactory;
import edu.uoc.pfc.mena.db.core.connection.factory.ConnexioFactory;
import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.db.utilities.provider.hsqldb.TaulaBDInfoHsqldbImpl;

public abstract class GeneradorTaulaBDInfoImpl implements GeneradorTaulaBDInfo, GeneradorConstants {
 
	  Logger log=Logger.getLogger(GeneradorTaulaBDInfoImpl.class.getName());
	
	  // Metadades BD
	  private DatabaseMetaData metadadaBD;
	  private String metadadaVersioBD;
	  private String metadadaNomDriver;
	  private int metadadaMajorVersioDriver;
	  private int metadadaMinorVersioDriver;
	   
	  // Mapping Java - BBDD
	  private String nomPackage;
	  private String directoriBase;
	  private String nomClasse;
	  private String nomFitxer;
	  private List<GeneradorColumnaBDInfo> infoColumnes;
	  private List<String> infoClaus;
	  
	  // Taula
	  private String nomTaula;
	  private String aliasTaula;
	  private int numCamps;
	  
	  // TODO: Son necessaris??
	  private String codiRaw;  
	  private String visibilitat;
	  private boolean isResetCampsActiu;
	  private String usuari;
	  
	  public Connection con;
	  
	  public GeneradorTaulaBDInfoImpl() throws Exception
	  {
		  con=ConnexioFactory.getInstance().creaConnexio();
		  this.init(con);
	  }
	  public GeneradorTaulaBDInfoImpl(Connection con) throws Exception
	  {
		  this.init(con);
	  }
	  
	  private void init(Connection con) throws Exception {
		  this.nomPackage = ConfiguradorFactory.getInstance().getPackageBDBeans();
		  this.directoriBase = ConfiguradorFactory.getInstance().getBasedirBDBeans();
		  		  
		  // Omplim les metadades
		  this.metadadaBD = con.getMetaData();
		  this.metadadaVersioBD=con.getMetaData().getDatabaseProductVersion();
		  this.metadadaNomDriver=con.getMetaData().getDriverName();
		  this.metadadaMajorVersioDriver=con.getMetaData().getDatabaseMajorVersion();
		  this.metadadaMinorVersioDriver=con.getMetaData().getDatabaseMinorVersion();
	   }
	  
	  public static void main(String args[]) throws Exception {
		  // init + connecct
		  GeneradorTaulaBDInfoImpl gd =  new TaulaBDInfoHsqldbImpl();
		  
		  // close + end
		  gd.con.close();
		  System.out.println("Test END!!");
		  
	  }
	 
	  public String getAliasTaula()
	  {
	    if ((this.aliasTaula != null) && (!this.aliasTaula.equals(""))) {
	      return this.aliasTaula;
	    }
	    return this.nomTaula;
	  }
	  
	  public void setAliasTaula(String alias){
		  this.aliasTaula=alias;
	  }
	  
	public boolean isResetCampsActiu() {
		return isResetCampsActiu;
	}
	public void setResetCampsActiu(boolean isResetCampsActiu) {
		this.isResetCampsActiu = isResetCampsActiu;
	}
	public String getNomClasse() {
		return nomClasse;
	}
	public void setNomClasse(String nomClasse) {
		this.nomClasse = nomClasse;
	}
	public String getNomFitxer() {
		return nomFitxer;
	}
	public void setNomFitxer(String nomFitxer) {
		this.nomFitxer = nomFitxer;
	}
	public String getVisibilitat() {
		return visibilitat;
	}
	public void setVisibilitat(String visibilitat) {
		this.visibilitat = visibilitat;
	}
	public String getUsuari() {
		return usuari;
	}
	public void setUsuari(String usuari) {
		this.usuari = usuari;
	}
	public String getNomTaula() {
		return nomTaula;
	}
	public void setNomTaula(String nomTaula) {
		this.nomTaula = nomTaula;
	}

	public String getNomPackage() {
		return nomPackage;
	}
	public void setNomPackage(String nomPackage) {
		this.nomPackage = nomPackage;
	}
	public String getDirectoriBase() {
		return directoriBase;
	}
	public void setDirectoriBase(String directoriBase) {
		this.directoriBase = directoriBase;
	}
	public DatabaseMetaData getMetadadaBD() {
		return metadadaBD;
	}
	public void setMetadadaBD(DatabaseMetaData metadadaBD) {
		this.metadadaBD = metadadaBD;
	}
	
	public String getMetadadaVersioBD() {
		return metadadaVersioBD;
	}
	public void setMetadadaVersioBD(String metadadaVersioBD) {
		this.metadadaVersioBD = metadadaVersioBD;
	}
	public String getMetadadaNomDriver() {
		return metadadaNomDriver;
	}
	public void setMetadadaNomDriver(String metadadaNomDriver) {
		this.metadadaNomDriver = metadadaNomDriver;
	}
	public int getMetadadaMajorVersioDriver() {
		return metadadaMajorVersioDriver;
	}
	public void setMetadadaMajorVersioDriver(int metadadaMajorVersioDriver) {
		this.metadadaMajorVersioDriver = metadadaMajorVersioDriver;
	}
	public int getMetadadaMinorVersioDriver() {
		return metadadaMinorVersioDriver;
	}
	public void setMetadadaMinorVersioDriver(int metadadaMinorVersioDriver) {
		this.metadadaMinorVersioDriver = metadadaMinorVersioDriver;
	}
	public List<GeneradorColumnaBDInfo> getInfoColumnes() {
		return infoColumnes;
	}
	public void setInfoColumnes(List<GeneradorColumnaBDInfo> columnaDada) {
		this.infoColumnes = columnaDada;
	}
	public List<String> getInfoClaus() {
		return infoClaus;
	}
	public void setInfoClaus(List<String> clauDada) {
		this.infoClaus = clauDada;
	}

	public String getCodiRaw() {
		return codiRaw;
	}
	
	public void setCodiRaw(String codiRaw) {
		this.codiRaw = codiRaw;
	}
	public Connection getCon() {
		return con;
	}
	public void setCon(Connection con) {
		this.con = con;
	}
	
	public int getNumCamps() {
		return numCamps;
	}
	public void setNumCamps(int numCamps) {
		this.numCamps = numCamps;
	}
	
	// TODO: Canviar el nom
	public abstract String getHasPK();
	
	public abstract void addPrimaryKey() throws BDException;
	
	public abstract void addColumnesInfo() throws BDException;
	 /* {
	    boolean bool = false;
	    this.infoColumnes = new Vector<GeneradorColumnaBDInfo>();
	    String str1 = this.isOracle() ? this.usuari.toUpperCase() : "%";
	    ResultSet localResultSet = this.metadadaBD.getColumns(null, str1, GeneradorUtils.toUpperCase(this, this.nomTaula), "%");
	    while (localResultSet.next())
	    {
	      //String str3 = localResultSet.getString(3);
	      //String str4 = localResultSet.getString(2);
	      String str5 = localResultSet.getString(4);
	      int j;
	      GeneradorColumnaBDInfo localColumnData;
	      if (this.isMySQL())
	      {
	        String str2 = localResultSet.getString(5);
	        j = localResultSet.getInt(7);
	        localColumnData = null;
	        try
	        {
	          localColumnData = new GeneradorColumnaBDInfo(str5, Integer.parseInt(str2), j);
	        }
	        catch (NumberFormatException localNumberFormatException)
	        {
	        	// TODO: Tracament errors
	          localNumberFormatException.printStackTrace(System.err);
	        }
	      }
	      else
	      {
	        int i;
	        if (this.isOracle())
	        {
	          i = localResultSet.getInt(5);
	          j = localResultSet.getInt(7);
	          if (localResultSet.getInt(9) > 0)
	            i = 8;
	          localColumnData = new GeneradorColumnaBDInfo(str5, i, j);
	        }
	        else
	        {
	          i = localResultSet.getInt(5);
	          j = localResultSet.getInt(7);
	          if (i == 2)
	            i = 3;
	          if ((localResultSet.getInt(9) > 0) && (i != 93) && (this.isSQLServer()))
	            i = 8;
	          localColumnData = new GeneradorColumnaBDInfo(str5, i, j);
	        }
	        
	        System.out.println(">>>>>>>>>>>>>> Info columna [ " + localColumnData+ "]");    
	      }
	      this.infoColumnes.addElement(localColumnData);
	    }
	    localResultSet.close();
	    this.numCamps = this.infoColumnes.size();
	    
	    
	    return bool;
	  }*/
	 
	 
	  public boolean hasClauPrimaria()
	  {
	    boolean bool = false;
	    this.setInfoClaus(new Vector<String>());
	    String str1 = getHasPK();
	    try
	    {
	      String str2 = null;
	      ResultSet localResultSet = this.getMetadadaBD().getPrimaryKeys(null, str1, this.getNomTaula());
	      while (localResultSet.next())
	      {
	        str2 = localResultSet.getString(4);
	        this.getInfoClaus().add(str2);
	        bool = true;
	      }
	      localResultSet.close();
	    }
	    catch (SQLException localSQLException)
	    {
	      String msg="Error obtenint les claus de la taula " + this.getNomTaula()+ " : " +localSQLException.getMessage();
	      log.error(msg);
	    }
	    return bool;
	  }
	  
	  // TODO: Son necessaris??
	@Override
	public Connection getConnexio() {
		// TODO Auto-generated method stub
		return null;
	}
	@Override
	public void setConnexio(Connection con) {
		// TODO Auto-generated method stub
		
	}



}
