package edu.uoc.pfc.mena.db.utilities;

import java.io.BufferedWriter;
import java.util.Date;
import java.util.Iterator;

import edu.uoc.pfc.mena.db.core.constants.Constants;
import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.db.core.util.Utils;

public abstract class GeneradorUtils extends Utils implements GeneradorConstants {
		
		public static String getClassNameFromDriver(String idDriver){
			return getClassNameFromDriver(idDriver, PREFIX_PCKG_DEFAULT, PREFIX_CLASS_DEFAULT, SUFIX_CLASS_DEFAULT);
		}
		
		public static String getClassNameFromDriver(String idDriver, String prefixPackage, String prefixClass, String sufixClass){
			String id=idDriver.toLowerCase().substring(0,1).toUpperCase().concat(idDriver.toLowerCase().substring(1));
			return prefixPackage.concat(idDriver.toLowerCase()).concat(DOT).concat(prefixClass).concat(id).concat(sufixClass);
		}

	  public static void writeHeader(BufferedWriter out, GeneradorTaulaBDInfo gd) throws BDException
	  {
			printBuffer(out,"",0);
		    printBuffer(out,"// GeneradorBDBeans Data: " + new Date(),0);
		    printBuffer(out,"package " + gd.getNomPackage() + ";",0);
		    printBuffer(out,"",0);
		    printBuffer(out,"import java.sql.*;",0);
		    printBuffer(out,"import edu.uoc.pfc.mena.db.core.annotations.*;",0);
		    printBuffer(out,"import java.io.Serializable;",0);
		    printBuffer(out,"",0);
		    printBuffer(out,"/**",0);
		    printBuffer(out,"  * " + gd.getNomClasse(),0);
		    printBuffer(out,"  * Taula: " + gd.getNomTaula(),0);
		    printBuffer(out,"  * -----------------------------------------",0);
		    printBuffer(out,"  * Versi base de dades : " + gd.getMetadadaVersioBD(),0);
		    printBuffer(out,"  * Nom driver : " + gd.getMetadadaNomDriver(),0);
		    printBuffer(out,"  * Versi driver : " + gd.getMetadadaMinorVersioDriver() + "." + gd.getMetadadaMajorVersioDriver(),0);
		    printBuffer(out,"  * Data generaci : " + new Date(),0);
		    printBuffer(out,"  * ",0);
		    printBuffer(out,"  */",0);
		    
		    // Indiquem l'annotacio de la taula
		    printBuffer(out,"@Entitat",0);
		    printBuffer(out,"@Taula(nom=\""+gd.getNomTaula()+"\")",0);
		    printBuffer(out,"public class " + gd.getNomClasse() + " implements Serializable {", 0);
		
		    printBuffer(out,"public static int numFields = " + gd.getNumCamps() + ";");
		    printBuffer(out,"private static final long serialVersionUID = 1L;");
		    printBuffer(out,"");
			    
		    // Fem el print de les variables de la classe
		    printBuffer(out," // Atributs de la classe");
		    for(GeneradorColumnaBDInfo gc: gd.getInfoColumnes()){
		    	printBuffer (out, gc.getTipusJava()+" " + toCamelCase(gc.getNom(), true)+ "; ");
		    }
		    printBuffer(out,"");
			    
		    writeConstructor(out, gd.getNomClasse());
	  }
	  
	  public static void writeConstructor(BufferedWriter out, String nomClasse) throws BDException
	  {
		  // Constructor buit
		  printBuffer(out,"public " + nomClasse + "() {}");
	  }
	  
	  public static void writeConstructor(BufferedWriter out, GeneradorTaulaBDInfo gd ) throws BDException
		{
			printBuffer(out,"public " + gd.getNomClasse() + "(", false);
		    Iterator<GeneradorColumnaBDInfo> localEnumeration = gd.getInfoColumnes().iterator();
		    GeneradorColumnaBDInfo localColumnData;
		    while (localEnumeration.hasNext())
		    {
		      localColumnData = localEnumeration.next();
		      printBuffer(out,localColumnData.getTipusJava() + " " + toCamelCase(localColumnData.getNom().toLowerCase(), true) + (localEnumeration.hasNext() ? "," : ""),0,false);
		    }
		    printBuffer(out,") {");
		    localEnumeration = gd.getInfoColumnes().iterator();
		    while (localEnumeration.hasNext())
		    {
		      localColumnData = localEnumeration.next();
		      printBuffer(out,"    this." + toCamelCase(localColumnData.getNom().toLowerCase(), true) + " = " + toCamelCase(localColumnData.getNom().toLowerCase(), true) + ";");
		    }
		    printBuffer(out,"  }");
	  }
	  
	  public static void writeMetodesUtils(BufferedWriter out) throws BDException{
		  printBuffer(out,"public static java.sql.Timestamp toTimestamp(String data) throws java.text.ParseException {");
		  printBuffer(out,"return toTimestamp(data, \"dd/MM/yyyy\");",3);
		  printBuffer(out,"}");
		
		  printBuffer(out,"public static java.sql.Timestamp toTimestamp(String data, String pattern) throws java.text.ParseException{");
		  printBuffer(out,"java.text.DateFormat dateFormat = new java.text.SimpleDateFormat(pattern);",3);
		  printBuffer(out,"java.util.Date date = dateFormat.parse(data);",3);
		  printBuffer(out,"long time = date.getTime();",3);
		  printBuffer(out,"return new java.sql.Timestamp(time);",3);
		  printBuffer(out,"}");
	 }
	public static void writeSetGet(BufferedWriter out, GeneradorTaulaBDInfo gt) throws BDException{
		for (GeneradorColumnaBDInfo c: gt.getInfoColumnes()){
			printBuffer(out,"");
			writeSetGetColumn(out, c);
		}
	}
		
	  public static void writeSetGetColumn(BufferedWriter out, GeneradorColumnaBDInfo columna ) throws BDException
	  {
		  try {
			    String str1 = toCamelCase(columna.getNom().toLowerCase(), true);
			    String str2 = toCamelCase(columna.getNom().toLowerCase(), true);
			    str2 = str2.substring(0, 1).toUpperCase() + str2.substring(1, str2.length());
			    
			    // Setter
			    printBuffer(out,"  /**");
			    printBuffer(out,"    * Sets del valor per " + str1);
			    printBuffer(out,"    * @param " + str1);
			    printBuffer(out,"    */");
			    printBuffer(out, "  public void set" + str2 + "(" + columna.getTipusJava() + " " + str1 + ") {");
			    printBuffer(out,"    this." + str1 + "=" + str1 + ";");
			    printBuffer(out,"  }");
			    printBuffer(out,"");
			    
			    // Getter
			    printBuffer(out,"  /**");
			    printBuffer(out,"    * Gets del valor per " + str1);
			    printBuffer(out,"    */");
			    // Afegim l'annotacio			    
			    if (columna.isAutoincremental()){
			    	String idAnnotation="  @Id(nom=\""+columna.getNom()+"\", tipus=\""+columna.getNomTipus()+"\")";
			    	printBuffer(out,idAnnotation);
			    } 
			    String annotationColumn=!columna.isAutoincremental()?"  @Columna(nom=\""+columna.getNom()+"\", tipus=\""+columna.getNomTipus()+"\", nullable=\""+(columna.isNullable()?"SI":"NO")+"\")":"@ValorGenerat";
			    printBuffer(out,annotationColumn);
			    printBuffer(out,"  public " + columna.getTipusJava() + " get" + str2 + "() {");
			    printBuffer(out,"    return " + str1 + ";");
			    printBuffer(out,"  }");

		  } catch(Exception e){
			  throw new BDException(Constants.GENERADOR_BEANS_ERROR_CODE, Constants.GENERADOR_BEANS_ERROR_CODE +" (getters/setters) : " + e.getMessage(), e);
		  }
	  }
 	 
	  private static void printBuffer(BufferedWriter out, String text, int tab, boolean ret) throws BDException {
		  try {
			  String prefix="";
			  for (int i=0;i<tab;i++){
				  prefix+="\t";
			  }
			  
			  String sufix=(ret==true)?"\n":"";
		      out.write(prefix+text + sufix);
		  }catch (Exception e){
			  throw new BDException(Constants.GENERADOR_BEANS_ERROR_CODE, Constants.GENERADOR_BEANS_ERROR_MSG + e.getMessage(),e);
		  }
	  }
	
	  private static void printBuffer(BufferedWriter out, String text, boolean b) throws BDException {
		  printBuffer(out, text, 1, b);
	  }
	  private static void printBuffer(BufferedWriter out, String text) throws BDException {
		  printBuffer(out, text, 1, true);
	  }
	  
	  private static void printBuffer(BufferedWriter out, String text, int tab) throws BDException {
		  printBuffer(out, text, tab, true);
	  }

	public static void writeEnd(BufferedWriter out) throws BDException {
		try {
			out.write("}");
			out.flush();
		} catch (Exception e) {
			throw new BDException(Constants.GENERADOR_BEANS_ERROR_CODE, Constants.GENERADOR_BEANS_ERROR_MSG + "(end class) :" + e.getMessage(),e);
		}
	}

	public static void writeHeader(BufferedWriter out, String pack,  String nomClasse) throws BDException {
		printBuffer(out,"package " + pack + ";",0);
		printBuffer(out,"import java.io.Serializable;",0);
		printBuffer(out,"");
	    printBuffer(out,"public class " + nomClasse + " implements Serializable {", 0);	
	    printBuffer(out,"");
	    printBuffer(out,"private static final long serialVersionUID = 1L;");
	    printBuffer(out,"");
	}
			 	
}
