package edu.uoc.pfc.mena.webapp.action;

import java.math.BigDecimal;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.webapp.beans.Desti;
import edu.uoc.pfc.mena.webapp.beans.TipusViatge;
import edu.uoc.pfc.mena.webapp.constants.ActionConstants;
import edu.uoc.pfc.mena.webapp.constants.AgenciaConstants;
import edu.uoc.pfc.mena.webapp.filter.OfertesFilter;
import edu.uoc.pfc.mena.webapp.form.OfertesForm;
import edu.uoc.pfc.mena.webapp.pojo.OfertaTO;
import edu.uoc.pfc.mena.webapp.services.DestinsServei;
import edu.uoc.pfc.mena.webapp.services.OfertesServei;
import edu.uoc.pfc.mena.webapp.services.TipusViatgesServei;
import edu.uoc.pfc.mena.webapp.spring.AgenciaContext;
import edu.uoc.pfc.mena.webapp.utils.StringUtils;


public class CercarOfertesAction extends GlobalAction  {
  
  private static final Logger log = Logger.getLogger( CercarOfertesAction.class );


  public ActionForward doExecute(ActionMapping mapping, ActionForm form, HttpServletRequest req, HttpServletResponse res) throws Exception{

	  try{
		  OfertesForm llistatForm= (OfertesForm) form; 
		  OfertesFilter filtre=this.formToFilter( llistatForm );
		  
		  
		 OfertesServei ofertesServei=AgenciaContext.getInstance().getOfertesServei();
		 List<OfertaTO> ofertesDisponibles= ofertesServei.cercarOfertes(filtre);
		 if (ofertesDisponibles!=null && ofertesDisponibles.size()==0){
			 ofertesDisponibles=null;
		 }
 
		 this.addDestins(req);
		 this.addTipusViatges(req);
	
		 req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_OFERTES_DISPONIBLES, ofertesDisponibles );
		 
		 // Afegim el filtre
		 req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_OFERTES_FILTRE, filtre );
		  
	  }catch(Exception e){
		  gestionarError(req,e);
		  return mapping.findForward( ActionConstants.MAPPING_ERROR); 
	  }
	  return mapping.findForward( ActionConstants.MAPPING_OK );
  }
 
  
  
  /**
   * Obte el filtre a partir del formulari
   */
  private OfertesFilter formToFilter( OfertesForm ofertesForm) {
	
	OfertesFilter filter = new OfertesFilter(  );
      
	if( !StringUtils.isNullOrEmpty( ofertesForm.getCerca() ) ){   
		  filter.setCerca(ofertesForm.getCerca() );          
    }
    
    if( !StringUtils.isNullOrEmpty( ofertesForm.getDesti() ) ){   
    	filter.setDesti( ofertesForm.getDesti() );          
    }
    if( ofertesForm.getTipus()!=null && ofertesForm.getTipus()!=-1){   
    	filter.setTipus( ofertesForm.getTipus() );          
    }
    
    if (ofertesForm.getId()!=null && ofertesForm.getId().intValue()>0){
    	filter.setId(ofertesForm.getId());
    }
    
    if (ofertesForm.getPreuMinim()>0){
    	filter.setPreuMinim(ofertesForm.getPreuMinim());
    }
    
    if (ofertesForm.getPreuMaxim()>0){
    	filter.setPreuMaxim(ofertesForm.getPreuMaxim());
    }
    if( !StringUtils.isNullOrEmpty( ofertesForm.getColumnSort() ) ){   
    	filter.setSortField(ofertesForm.getColumnSort() );          
    }
    
    if( !StringUtils.isNullOrEmpty( ofertesForm.getSortOrder() ) ){   
    	filter.setSortOrder (ofertesForm.getSortOrder() );          
    }
    
    return filter;
  }  
  
  
  protected OfertaTO getOferta(HttpServletRequest req) {
		
		OfertaTO o= new OfertaTO();
		
		// fem els checks dels camps obligatoris
		super.checkInformat(req.getParameter("oferta_codi"), AgenciaConstants.OFERTA_CODI_REQUIRED, req.getLocale());
		super.checkBigDecimalInformat(req.getParameter("oferta_places"), AgenciaConstants.OFERTA_PLACES_REQUIRED, req.getLocale());
		super.checkBigDecimalInformat(req.getParameter("oferta_preu"), AgenciaConstants.OFERTA_PREU_REQUIRED, req.getLocale());
		super.checkSelect(req.getParameter("oferta_desti"), AgenciaConstants.OFERTA_DESTI_REQUIRED, req.getLocale());
		super.checkSelect(req.getParameter("oferta_tipus_viatge"), AgenciaConstants.OFERTA_TIPUS_VIATGE_REQUIRED, req.getLocale());
		
		if(!StringUtils.isNullOrEmpty(req.getParameter("oferta_id"))){
			o.getOferta().setId(Integer.parseInt(req.getParameter("oferta_id")));
		}
		
		o.getOferta().setCodi(req.getParameter("oferta_codi"));
		o.getOferta().setDescripcio(req.getParameter("oferta_descripcio"));
		
		if(!StringUtils.isNullOrEmpty(req.getParameter("oferta_places"))){
			o.getOferta().setPlaces(Integer.parseInt(req.getParameter("oferta_places")));
		}
		if(!StringUtils.isNullOrEmpty(req.getParameter("oferta_preu"))){
			o.getOferta().setPreu(new BigDecimal(req.getParameter("oferta_preu")));
		}
		if(!StringUtils.isNullOrEmpty(req.getParameter("oferta_desti"))){
			o.getDesti().setId(Integer.parseInt(req.getParameter("oferta_desti")));
		}
		if(!StringUtils.isNullOrEmpty(req.getParameter("oferta_tipus_viatge"))){
			o.getTipus().setId(Integer.parseInt(req.getParameter("oferta_tipus_viatge")));
		}
		return o;
	  }
    
}