package edu.uoc.pfc.mena.webapp.action;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.webapp.beans.Desti;
import edu.uoc.pfc.mena.webapp.beans.TipusViatge;
import edu.uoc.pfc.mena.webapp.constants.ActionConstants;
import edu.uoc.pfc.mena.webapp.constants.AgenciaConstants;
import edu.uoc.pfc.mena.webapp.exception.AgenciaException;
import edu.uoc.pfc.mena.webapp.services.DestinsServei;
import edu.uoc.pfc.mena.webapp.services.TipusViatgesServei;
import edu.uoc.pfc.mena.webapp.spring.AgenciaContext;
import edu.uoc.pfc.mena.webapp.utils.MessageResourcesUtil;

public abstract class GlobalAction extends Action {
	private static final Logger log = Logger.getLogger( GlobalAction.class );
	protected List<String> errorMessages= new ArrayList<String>();
	
	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest req, HttpServletResponse res) throws Exception{

		return this.doExecute(mapping, form, req, res);
	 }
	
	protected abstract ActionForward doExecute(ActionMapping mapping, ActionForm form, HttpServletRequest req, HttpServletResponse res) throws Exception;
	
	protected String getMissatgeError (){
		String missatgeError = "";
		for(String message: errorMessages){
			missatgeError = missatgeError + "<p><img src=\"/cmeni-pfc-webapp/content/img/icoTitolError.gif\"/>&nbsp" + message + "</p>\n";
		}		
		return missatgeError;
	}
	
	
	protected void checkSelect (String value, String messageKey, Locale locale){
		if (null == value || "".equals(value) || AgenciaConstants.SELECT_OPTION_DEFECTE_VALUE.equals(value)){
			this.errorMessages.add(MessageResourcesUtil.getDisplayString(messageKey, locale));		
	  	}
	}
	
	protected void checkInformat (String value, String messageKey, Locale locale){
		if (null == value || "".equals(value.trim())){
			this.errorMessages.add(MessageResourcesUtil.getDisplayString(messageKey, locale));    			
		}
	}
	
	protected void checkBigDecimalInformat (String value, String messageKey, Locale locale){
		if (null == value || "".equals(value.trim()) || (new BigDecimal(value)).intValue()<=0){
			this.errorMessages.add(MessageResourcesUtil.getDisplayString(messageKey, locale));    			
		}
	}
	
	protected void gestionarError(HttpServletRequest req, Exception e){
		 
		  try {
			  String msg=MessageResourcesUtil.getDisplayString(AgenciaConstants.ERROR_MSG_GENERAL, req.getLocale());		  
			  if (e instanceof BDException || e instanceof AgenciaException){
				  msg=e.getMessage();
			  }
			  this.errorMessages.add(msg);
			  req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_ERROR, this.getMissatgeError());
			  
			  // Afegim destins i tipus de viatges
			  this.addDestins(req);
			  this.addTipusViatges(req);
		  }catch(Exception exc){
			  log.error(exc.getMessage());
		  }
		  	  
	}
	
	protected void addDestins(HttpServletRequest req) throws BDException{
		  // Obtenim tots els destins i tipus de viatges disponibles
		  DestinsServei destinsServei=AgenciaContext.getInstance().getDestinsServei();
		  List<Desti> destins= destinsServei.cercarTotsDestins();
		  req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_DESTINS, destins );
		  	  
	  	}
	  
	  	protected void addTipusViatges(HttpServletRequest req) throws BDException{
		  TipusViatgesServei tipusViatgesServei=AgenciaContext.getInstance().getTipusViatgesServei();
		  List<TipusViatge> tipusViatges= tipusViatgesServei.cercarTotsTipusViatge();
		  req.setAttribute( ActionConstants.HTTP_REQUEST_ATTRIBUTE_TIPUS_VIATGE, tipusViatges );
		  	  
	  	}
}
