package edu.uoc.pfc.mena.webapp.dao;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import edu.uoc.pfc.mena.db.core.exception.BDException;
import edu.uoc.pfc.mena.db.core.session.Sessio;
import edu.uoc.pfc.mena.db.core.session.factory.SessioFactory;


public abstract class AbstractGenericDAO<E> {
	
	private static final Logger log = Logger.getLogger( AbstractGenericDAO.class );
	
  /**
   * 
   */
  protected AbstractGenericDAO() {
    super();
  }

  protected Sessio obreTransaccio(boolean autocommit) throws BDException{
	  Sessio sessio= SessioFactory.getInstance().newSessio();
	  sessio.newTransaccio(autocommit);
	  log.info("S'ha obert correctament una nova transacci amb autocommit a "+autocommit);
	  return sessio;
  }
  
  protected int insert(Sessio sessio, E objecte) throws BDException{
	  int id=sessio.insert(objecte);
	  log.debug("Mtode insert executat correctament per l'objecte "+objecte.getClass()+". L'identificador generat s: "+id);
	  return id;
  }	
  
  protected int insert(E objecte, boolean autocommit) throws BDException{
	  return insert(obreTransaccio(autocommit), objecte);
  }	
  
  protected void update(Sessio sessio, E objecte) throws BDException{
	  sessio.update(objecte);
	  log.debug("Mtode update executat correctament per l'objecte "+objecte.getClass());
  }	
  
  protected void update(E objecte, boolean autocommit) throws BDException{
	  update(obreTransaccio(autocommit), objecte);	  
  }
  

  protected void delete(E objecte, boolean autocommit) throws BDException {
	  delete(obreTransaccio(autocommit), objecte);
  }
  
  protected void delete(Sessio sessio, E objecte) throws BDException{
	  sessio.delete(objecte);
	  log.debug("Mtode delete executat correctament per l'objecte "+objecte.getClass());
  }	
  
  @SuppressWarnings("unchecked")
  protected List<E> find (E objecte) throws BDException{
	  Sessio sessio= SessioFactory.getInstance().newSessio();
	  sessio.newTransaccio();
	  if (objecte!=null){
		  log.debug("Executant mtode find amb per a l'objecte amb classe["+objecte.getClass()+"]");
	  }
	  List<E> resp=(List<E>) sessio.find(objecte);
	  if (resp==null){
		  log.debug("No s'ha obtingut cap resultat a la cerca!");
		  return new ArrayList<E>();
	  }
	  
	  return resp;
  }

  @SuppressWarnings("unchecked")
  protected E findById (E objecte) throws BDException {
	  Sessio sessio= SessioFactory.getInstance().newSessio();
	  sessio.newTransaccio();
	  if (objecte!=null){
		  log.debug("Executant mtode findById amb per a l'objecte amb classe["+objecte.getClass()+"]");
	  }
	  List<E> resp=(List<E>) sessio.find(objecte);
	  if (resp==null){
		  throw new BDException(1,"La cerca per clau primaria no ha donat cap resultat");
	  }
	  return resp.get(0);
  }
  
  @SuppressWarnings("unchecked")
  protected List<E> find(String sql, E objecte) throws BDException {
	  Sessio sessio= SessioFactory.getInstance().newSessio();
	  sessio.newTransaccio();
	  log.debug("Executant mtode find amb per la consulta ["+sql+"]");
	  List<E> resp=(List<E>) sessio.find(sql, objecte);
	  
	  if (resp==null){
		  log.debug("No s'ha obtingut cap resultat a la cerca!");
		  return new ArrayList<E>();
	  }
	  
	  return resp;
  }
  
  protected List<Object> executeQuery(String sql, Class<?> c) throws BDException, InstantiationException, IllegalAccessException {
	  Sessio sessio= SessioFactory.getInstance().newSessio();
	  sessio.newTransaccio();
	  log.debug("Executant mtode find amb per la consulta ["+sql+"]");
	  List<Object> resp=(List<Object>) sessio.find(sql, c.newInstance());
	  
	  if (resp==null){
		  log.debug("No s'ha obtingut cap resultat a la cerca!");
		  return new ArrayList<Object>();
	  }
	  
	  return resp;  
  }
  
  protected void tancaTransaccio(Sessio sessio) throws BDException{
	  sessio.tancaTransaccio();
	  log.debug("Sessi tancada correctament!");
  }
  
 
  protected int calculateTotalPages(long totalRecords, int recordsPerPage){
    return totalRecords > 0 && recordsPerPage > 0 ? (int) Math.floor(totalRecords/recordsPerPage)+ ( totalRecords%recordsPerPage > 0 ? 1 : 0 ) : 1;
  }
  
}
