package edu.uoc.pfc.mena.webapp.services.provider.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.log4j.Logger;

import edu.uoc.pfc.mena.webapp.services.provider.PropertiesProviderService;
import edu.uoc.pfc.mena.webapp.utils.FileUtils;





public class PropertiesProviderServiceImpl implements PropertiesProviderService {

  private static final String CLASSPATH_TOKEN = "classpath:";

  private static final Logger log = Logger.getLogger( PropertiesProviderServiceImpl.class );

  
  
  private Properties cfg;
  
  private String propertiesFilename;

  public PropertiesProviderServiceImpl() {
    super();
  }

  public PropertiesProviderServiceImpl(String propertiesFilename) {
    this();
    this.setPropertiesFilename( propertiesFilename );
  }

  
  

  public String getProperty(String propName) {
    if( this.cfg == null ){
      throw new RuntimeException("No existeix arxiu de properties configurat");
    }
    String out = this.cfg.getProperty( propName );    
    return out!=null ? out.trim() : out;
  }

  public int getIntProperty(String propName) {
    return Integer.parseInt( this.getProperty( propName ) );
  }

  public long getLongProperty(String propName) {
    return Long.parseLong( this.getProperty( propName ) );
  }

  public boolean getBooleanProperty(String propName) {
    return Boolean.parseBoolean( this.getProperty( propName ) );
  }
  
  /**
   * Obte el nom de l'arxiu de properties. 
   * @return
   */
  public String getPropertiesFilename(){
    return this.propertiesFilename;
  }

  /**
   * @param propertiesFilename the propertiesFilename to set
   */
  public void setPropertiesFilename(String propertiesFilename) {
    this.propertiesFilename = propertiesFilename;
    this.config();
  }

  
  
  
  /**
   * Carrega l'arxiu de properties. Usarà el mètode getPropertiesFilename per obtenir el nom de l'arxiu
   */
  private void config() {    
    InputStream is = this.getPropertiesInputStream();
    if( is != null ){
      this.config( is );
    }else{
      String msgError = "Error al carregar las properties del arxiu [" + this.getPropertiesFilename() + "]";
      throw new RuntimeException( msgError );
    }
  }// config



  /**
   * 
   * @param is
   */
  private void config(InputStream is) {
    this.cfg = new Properties();
    try{
      this.cfg.load( is );
    }catch ( IOException e ){
      String msgError = "Error al carregar las properties del arxiu [" + this.getPropertiesFilename() + "]";
      log.error( msgError, e );
      throw new RuntimeException( msgError, e );
    }    
    log.info( "Properties carregades correctament [" + this.getPropertiesFilename() + "] " );
  }// config

  

  
  
  /**
   * @return
   */
  private InputStream getPropertiesInputStream() {
    InputStream is = null;
    String filename = this.getPropertiesFilename();
    if( filename!=null && filename.trim().length() >  0){
      if( filename.startsWith( CLASSPATH_TOKEN ) ){       
        is = FileUtils.getFileAsStreamFromClassLoader( filename.substring( CLASSPATH_TOKEN.length() ) );    
      }else{
        is = FileUtils.getFileAsStream( filename );
      }
    }
    return is;
  }

  
}