package edu.uoc.pfc.mena.webapp.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import org.apache.log4j.Logger;


public class DateUtils {
	
  private static final Logger log = Logger.getLogger( DateUtils.class );

  
  public static String formatDate(Date date, String pattern){
    if (pattern == null ) {
      throw new IllegalArgumentException("El patró per formatar les dates no pot ser nul");
    }
    if( date!=null ){
      SimpleDateFormat sdf = new SimpleDateFormat( pattern );
      return sdf.format( date );
    }
    return "";
  }


  public static String formatDate(Date date, String pattern, Locale locale){
    if (pattern == null || locale==null) {
      throw new IllegalArgumentException("El patró i el locale per formatar les dates no pot ser nul");
    }
    if( date!=null ){
      SimpleDateFormat sdf = new SimpleDateFormat( pattern, locale );
      return sdf.format( date );
    }
    return "";
  }
  

  public static Date parseDate(String dataEstat, String pattern) {
    if (pattern == null ) {
      throw new IllegalArgumentException("El patró per formatar les dates no pot ser nul");
    }
    if( dataEstat!=null && dataEstat.trim().length() > 0){
      SimpleDateFormat sdf = new SimpleDateFormat( pattern );
      try{
        return sdf.parse( dataEstat );
      }catch ( ParseException e ){
        log.warn( "Error en fer parse de la data [" + dataEstat + "] amb el patró [" + pattern + "]", e );
      }
    }
    return null;
  }


  private static Calendar getCalendar(Date date) {
    if (date == null ) {
      throw new IllegalArgumentException("La data no pot ser nul·la");
    }
    Calendar cal1 = Calendar.getInstance();
    cal1.setTime(date);
    return cal1;
  }


  public static boolean isSameDay(Date date1, Date date2) {
      return isSameDay( getCalendar( date1 ), getCalendar( date2 ));
  }



  public static boolean isSameDay(Calendar cal1, Calendar cal2) {
      if (cal1 == null || cal2 == null) {
        throw new IllegalArgumentException("Les dates no poden ser nul·les");
      }
      return (cal1.get(Calendar.ERA) == cal2.get(Calendar.ERA) &&
              cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
              cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR));
  }

  

  public static boolean isSameWeek(Date date1, Date date2) {
      return isSameWeek( getCalendar( date1 ), getCalendar( date2 ));
  }
  

  public static boolean isSameWeek(Calendar cal1, Calendar cal2) {
      if (cal1 == null || cal2 == null) {
        throw new IllegalArgumentException("Les dates no poden ser nul·les");
      }
      return (cal1.get(Calendar.ERA) == cal2.get(Calendar.ERA) &&
              cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
              cal1.get(Calendar.WEEK_OF_YEAR) == cal2.get(Calendar.WEEK_OF_YEAR));
  }
  

  public static boolean isToday(Date date) {
      return isSameDay(date, Calendar.getInstance().getTime());
  }
  

  public static boolean isThisWeek(Date date) {
      return isSameWeek(date, Calendar.getInstance().getTime());
  }
  

  public static Date addSeconds(Date date, int seconds){
    return addTime( date, Calendar.SECOND, seconds );
  }

  
  public static Date addMinutes(Date date, int minutes){
    return addTime( date, Calendar.MINUTE, minutes );
  }
  

  public static Date addTime(Date date, int field, int quantity){
    Calendar cal = getCalendar( date );
    cal.add( field, quantity );
    Date out = cal.getTime();

    return out;
  }
    
}// DateUtils
